/*
 * #%L
 * OME-XML C++ library for working with OME-XML metadata structures.
 * %%
 * Copyright © 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*─────────────────────────────────────────────────────────────────────────────
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *─────────────────────────────────────────────────────────────────────────────
 */

#ifndef OME_XML_MODEL_DETECTORSETTINGS_H
#define OME_XML_MODEL_DETECTORSETTINGS_H

#include <algorithm>
#include <list>
#include <stdexcept>
#include <string>
#include <vector>

#include <ome/common/log.h>

#include <ome/common/xml/dom/Document.h>
#include <ome/common/xml/dom/Element.h>
#include <ome/common/xml/dom/Node.h>

#include <ome/xml/model/primitives/Quantity.h>

#include <ome/xml/model/Detector.h>
#include <ome/xml/model/Settings.h>
#include <ome/xml/model/enums/Binning.h>
#include <ome/xml/model/enums/UnitsElectricPotential.h>
#include <ome/xml/model/enums/UnitsFrequency.h>
#include <ome/xml/model/primitives/PositiveInteger.h>

namespace ome
{
  namespace xml
  {
    namespace model
    {

      // Forward declarations.
      class OMEModel;

      /**
       * DetectorSettings model object.
       */
      class DetectorSettings : public Settings
      {
      private:
        class Impl;
        /// Private implementation details.
        std::shared_ptr<Impl> impl;

      public:
        /// Default constructor.
        DetectorSettings();

        /**
         * Copy constructor.
         *
         * @param copy the DetectorSettings to copy.
         */
        DetectorSettings (const DetectorSettings& copy);

        /// Destructor.
        virtual
        ~DetectorSettings ();

        /**
         * Create a DetectorSettings model object from DOM element.
         *
         * @param element root of the XML DOM tree to from which to
         * construct the model object graph.
         * @param model handler for the OME model used to track
         * instances and references seen during the update.
         * @throws EnumerationException if there is an error
         * instantiating an enumeration during model object creation,
         * or ModelException if there are any consistency or validity
         * errors found during processing.
         *
         * @returns a new model object.
         */
        static std::shared_ptr<DetectorSettings>
        create(const common::xml::dom::Element& element,
               ome::xml::model::OMEModel& model);

        // Documented in superclass.
        const std::string&
        elementName() const;

        // Documented in superclass.
        bool
        validElementName(const std::string& name) const;

      private:
        /// Assignment operator (deleted).
        DetectorSettings&
        operator= (const DetectorSettings&);

      public:
        // -- OMEModelObject API methods --

        /// @copydoc ome::xml::model::OMEModelObject::update
        virtual void
        update(const common::xml::dom::Element&  element,
               ome::xml::model::OMEModel& model);

      public:
        // -- DetectorSettings API methods --

        /// @copydoc ome::xml::model::OMEModelObject::link
        bool
        link (std::shared_ptr<Reference>& reference,
              std::shared_ptr<ome::xml::model::OMEModelObject>& object);

        /**
         * Get the Zoom property.
         *
         * @returns the Zoom property.
         */
        std::shared_ptr<double>
        getZoom ();

        /**
         * Get the Zoom property.
         *
         * @returns the Zoom property.
         */
        const std::shared_ptr<double>
        getZoom () const;

        /**
         * Set the Zoom property.
         *
         * @param zoom the value to set.
         */
        void
        setZoom (std::shared_ptr<double>& zoom);

        /**
         * Get the Binning property.
         *
         * @returns the Binning property.
         */
        std::shared_ptr<ome::xml::model::enums::Binning>
        getBinning ();

        /**
         * Get the Binning property.
         *
         * @returns the Binning property.
         */
        const std::shared_ptr<ome::xml::model::enums::Binning>
        getBinning () const;

        /**
         * Set the Binning property.
         *
         * @param binning the value to set.
         */
        void
        setBinning (std::shared_ptr<ome::xml::model::enums::Binning>& binning);

        /**
         * Get the Integration property.
         *
         * @returns the Integration property.
         */
        std::shared_ptr<ome::xml::model::primitives::PositiveInteger>
        getIntegration ();

        /**
         * Get the Integration property.
         *
         * @returns the Integration property.
         */
        const std::shared_ptr<ome::xml::model::primitives::PositiveInteger>
        getIntegration () const;

        /**
         * Set the Integration property.
         *
         * @param integration the value to set.
         */
        void
        setIntegration (std::shared_ptr<ome::xml::model::primitives::PositiveInteger>& integration);

        /**
         * Get the ReadOutRate property.
         *
         * @returns the ReadOutRate property.
         */
        std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsFrequency > >&
        getReadOutRate ();

        /**
         * Get the ReadOutRate property.
         *
         * @returns the ReadOutRate property.
         */
        const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsFrequency > >&
        getReadOutRate () const;

        /**
         * Set the ReadOutRate property.
         *
         * @param readOutRate the value to set.
         */
        void
        setReadOutRate (const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsFrequency > >& readOutRate);

        /**
         * Get the Gain property.
         *
         * @returns the Gain property.
         */
        std::shared_ptr<double>
        getGain ();

        /**
         * Get the Gain property.
         *
         * @returns the Gain property.
         */
        const std::shared_ptr<double>
        getGain () const;

        /**
         * Set the Gain property.
         *
         * @param gain the value to set.
         */
        void
        setGain (std::shared_ptr<double>& gain);

        /**
         * Get the Offset property.
         *
         * @returns the Offset property.
         */
        std::shared_ptr<double>
        getOffset ();

        /**
         * Get the Offset property.
         *
         * @returns the Offset property.
         */
        const std::shared_ptr<double>
        getOffset () const;

        /**
         * Set the Offset property.
         *
         * @param offset the value to set.
         */
        void
        setOffset (std::shared_ptr<double>& offset);

        /**
         * Get the ID property.
         *
         * @returns the ID property.
         */
        const std::string&
        getID () const;

        /**
         * Set the ID property.
         *
         * @param id the value to set.
         */
        void
        setID (const std::string& id);

        /**
         * Get the Voltage property.
         *
         * @returns the Voltage property.
         */
        std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsElectricPotential > >&
        getVoltage ();

        /**
         * Get the Voltage property.
         *
         * @returns the Voltage property.
         */
        const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsElectricPotential > >&
        getVoltage () const;

        /**
         * Set the Voltage property.
         *
         * @param voltage the value to set.
         */
        void
        setVoltage (const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsElectricPotential > >& voltage);

        /**
         * Get the Detector property.
         *
         * @returns the Detector property.
         */
        std::weak_ptr<ome::xml::model::Detector>
        getDetector ();

        /**
         * Get the Detector property.
         *
         * @returns the Detector property.
         */
        const std::weak_ptr<ome::xml::model::Detector>
        getDetector () const;

        /**
         * Set the Detector property.
         *
         * @param detector the value to set.
         */
        void
        setDetector (std::weak_ptr<ome::xml::model::Detector>& detector);

      protected:
        // Documented in base class.
        virtual void
        asXMLElementInternal (common::xml::dom::Document& document,
                              common::xml::dom::Element&  element) const;

      public:
       // Documented in superclass.
       const std::string&
       getXMLNamespace() const;
      };

    }
  }
}

#endif // OME_XML_MODEL_DETECTORSETTINGS_H

/*
 * Local Variables:
 * mode:C++
 * End:
 */
