/*
 * #%L
 * OME-XML C++ library for working with OME-XML metadata structures.
 * %%
 * Copyright © 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*─────────────────────────────────────────────────────────────────────────────
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *─────────────────────────────────────────────────────────────────────────────
 */

#ifndef OME_XML_MODEL_LEADER_H
#define OME_XML_MODEL_LEADER_H

#include <algorithm>
#include <list>
#include <stdexcept>
#include <string>
#include <vector>

#include <ome/common/log.h>

#include <ome/common/xml/dom/Document.h>
#include <ome/common/xml/dom/Element.h>
#include <ome/common/xml/dom/Node.h>

#include <ome/xml/model/primitives/Quantity.h>

#include <ome/xml/model/Reference.h>

namespace ome
{
  namespace xml
  {
    namespace model
    {

      // Forward declarations.
      class ExperimenterGroup;
      class OMEModel;

      /**
       * Leader model object.
       */
      class Leader : public Reference
      {
      private:
        class Impl;
        /// Private implementation details.
        std::shared_ptr<Impl> impl;

      public:
        /// Default constructor.
        Leader();

        /**
         * Copy constructor.
         *
         * @param copy the Leader to copy.
         */
        Leader (const Leader& copy);

        /// Destructor.
        virtual
        ~Leader ();

        /**
         * Create a Leader model object from DOM element.
         *
         * @param element root of the XML DOM tree to from which to
         * construct the model object graph.
         * @param model handler for the OME model used to track
         * instances and references seen during the update.
         * @throws EnumerationException if there is an error
         * instantiating an enumeration during model object creation,
         * or ModelException if there are any consistency or validity
         * errors found during processing.
         *
         * @returns a new model object.
         */
        static std::shared_ptr<Leader>
        create(const common::xml::dom::Element& element,
               ome::xml::model::OMEModel& model);

        // Documented in superclass.
        const std::string&
        elementName() const;

        // Documented in superclass.
        bool
        validElementName(const std::string& name) const;

      private:
        /// Assignment operator (deleted).
        Leader&
        operator= (const Leader&);

      public:
        // -- OMEModelObject API methods --

        /// @copydoc ome::xml::model::OMEModelObject::update
        virtual void
        update(const common::xml::dom::Element&  element,
               ome::xml::model::OMEModel& model);

      public:
        // -- Leader API methods --

        /// @copydoc ome::xml::model::OMEModelObject::link
        bool
        link (std::shared_ptr<Reference>& reference,
              std::shared_ptr<ome::xml::model::OMEModelObject>& object);

        /**
         * Get the ID property.
         *
         * @returns the ID property.
         */
        const std::string&
        getID () const;

        /**
         * Set the ID property.
         *
         * @param id the value to set.
         */
        void
        setID (const std::string& id);

        /**
         * Get size of linked ExperimenterGroup list.
         *
         * @returns the size of the list.
         */
        OMEModelObject::indexed_container<ome::xml::model::ExperimenterGroup, std::weak_ptr>::type::size_type
        sizeOfLinkedExperimenterGroupList () const;

        /**
         * Get the linked ExperimenterGroup list.
         *
         * @returns a reference to the list.
         */
        OMEModelObject::indexed_container<ome::xml::model::ExperimenterGroup, std::weak_ptr>::type
        getLinkedExperimenterGroupList () const;

        /**
         * Get linked ExperimenterGroup.
         *
         * @param index the index number of the ExperimenterGroup.
         * @returns a weak pointer to the ExperimenterGroup.
         * @throws std::out_of_range if the index is invalid.
         */
        const std::weak_ptr<ome::xml::model::ExperimenterGroup>&
        getLinkedExperimenterGroup (OMEModelObject::indexed_container<ome::xml::model::ExperimenterGroup, std::weak_ptr>::type::size_type index) const;

        /**
         * Set linked ExperimenterGroup.
         *
         * @note The index must be valid.
         *
         * @param index the index number of the ExperimenterGroup.
         * @param experimenterGroup_BackReference the ExperimenterGroup to set.
         * @returns a weak pointer to the ExperimenterGroup.
         * @throws std::out_of_range if the index is invalid.
         */
        const std::weak_ptr<ome::xml::model::ExperimenterGroup>&
        setLinkedExperimenterGroup (OMEModelObject::indexed_container<ome::xml::model::ExperimenterGroup, std::weak_ptr>::type::size_type index,
                                     const std::shared_ptr<ome::xml::model::ExperimenterGroup>& experimenterGroup_BackReference);

        /**
         * Link ExperimenterGroup.
         *
         * @param experimenterGroup_BackReference the ExperimenterGroup to link.
         * @returns @c true if the object was added to the internal
         * experimenterGroups list, otherwise @c false.
         *
         * @todo Why have a return value here; what is it used for?
         * Is this an artifact of the Java API?
         */
        bool
        linkExperimenterGroup (const std::shared_ptr<ome::xml::model::ExperimenterGroup>& experimenterGroup_BackReference);

        /**
         * Unlink ExperimenterGroup.
         *
         * @param experimenterGroup_BackReference the ExperimenterGroup to unlink.
         *
         * @returns @c true if the ExperimenterGroup was unlinked, otherwise
         * @c false if the ExperimenterGroup was not linked and could hence not be
         * unlinked.
         *
         * @todo Does the return value serve any useful purpose?  Is
         * this also inherited Java API?
         * @todo Doesn't the removal break the indexing?
         * @todo Does the removal deal with multiple instances; we
         * aren't preventing duplicates on insertion.
         */
        bool
        unlinkExperimenterGroup (const std::shared_ptr<ome::xml::model::ExperimenterGroup>& experimenterGroup_BackReference);

      protected:
        // Documented in base class.
        virtual void
        asXMLElementInternal (common::xml::dom::Document& document,
                              common::xml::dom::Element&  element) const;

      public:
       // Documented in superclass.
       const std::string&
       getXMLNamespace() const;
      };

    }
  }
}

#endif // OME_XML_MODEL_LEADER_H

/*
 * Local Variables:
 * mode:C++
 * End:
 */
