/*
 * #%L
 * OME-XML C++ library for working with OME-XML metadata structures.
 * %%
 * Copyright © 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*─────────────────────────────────────────────────────────────────────────────
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *─────────────────────────────────────────────────────────────────────────────
 */

#ifndef OME_XML_MODEL_LIGHTSOURCESETTINGS_H
#define OME_XML_MODEL_LIGHTSOURCESETTINGS_H

#include <algorithm>
#include <list>
#include <stdexcept>
#include <string>
#include <vector>

#include <ome/common/log.h>

#include <ome/common/xml/dom/Document.h>
#include <ome/common/xml/dom/Element.h>
#include <ome/common/xml/dom/Node.h>

#include <ome/xml/model/primitives/Quantity.h>

#include <ome/xml/model/LightSource.h>
#include <ome/xml/model/MicrobeamManipulation.h>
#include <ome/xml/model/Settings.h>
#include <ome/xml/model/enums/UnitsLength.h>
#include <ome/xml/model/primitives/PercentFraction.h>
#include <ome/xml/model/primitives/PositiveFloat.h>

namespace ome
{
  namespace xml
  {
    namespace model
    {

      // Forward declarations.
      class OMEModel;

      /**
       * LightSourceSettings model object.
       */
      class LightSourceSettings : public Settings
      {
      private:
        class Impl;
        /// Private implementation details.
        std::shared_ptr<Impl> impl;

      public:
        /// Default constructor.
        LightSourceSettings();

        /**
         * Copy constructor.
         *
         * @param copy the LightSourceSettings to copy.
         */
        LightSourceSettings (const LightSourceSettings& copy);

        /// Destructor.
        virtual
        ~LightSourceSettings ();

        /**
         * Create a LightSourceSettings model object from DOM element.
         *
         * @param element root of the XML DOM tree to from which to
         * construct the model object graph.
         * @param model handler for the OME model used to track
         * instances and references seen during the update.
         * @throws EnumerationException if there is an error
         * instantiating an enumeration during model object creation,
         * or ModelException if there are any consistency or validity
         * errors found during processing.
         *
         * @returns a new model object.
         */
        static std::shared_ptr<LightSourceSettings>
        create(const common::xml::dom::Element& element,
               ome::xml::model::OMEModel& model);

        // Documented in superclass.
        const std::string&
        elementName() const;

        // Documented in superclass.
        bool
        validElementName(const std::string& name) const;

      private:
        /// Assignment operator (deleted).
        LightSourceSettings&
        operator= (const LightSourceSettings&);

      public:
        // -- OMEModelObject API methods --

        /// @copydoc ome::xml::model::OMEModelObject::update
        virtual void
        update(const common::xml::dom::Element&  element,
               ome::xml::model::OMEModel& model);

      public:
        // -- LightSourceSettings API methods --

        /// @copydoc ome::xml::model::OMEModelObject::link
        bool
        link (std::shared_ptr<Reference>& reference,
              std::shared_ptr<ome::xml::model::OMEModelObject>& object);

        /**
         * Get the Wavelength property.
         *
         * @returns the Wavelength property.
         */
        std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsLength, ome::xml::model::primitives::PositiveFloat > >&
        getWavelength ();

        /**
         * Get the Wavelength property.
         *
         * @returns the Wavelength property.
         */
        const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsLength, ome::xml::model::primitives::PositiveFloat > >&
        getWavelength () const;

        /**
         * Set the Wavelength property.
         *
         * @param wavelength the value to set.
         */
        void
        setWavelength (const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsLength, ome::xml::model::primitives::PositiveFloat > >& wavelength);

        /**
         * Get the Attenuation property.
         *
         * @returns the Attenuation property.
         */
        std::shared_ptr<ome::xml::model::primitives::PercentFraction>
        getAttenuation ();

        /**
         * Get the Attenuation property.
         *
         * @returns the Attenuation property.
         */
        const std::shared_ptr<ome::xml::model::primitives::PercentFraction>
        getAttenuation () const;

        /**
         * Set the Attenuation property.
         *
         * @param attenuation the value to set.
         */
        void
        setAttenuation (std::shared_ptr<ome::xml::model::primitives::PercentFraction>& attenuation);

        /**
         * Get the ID property.
         *
         * @returns the ID property.
         */
        const std::string&
        getID () const;

        /**
         * Set the ID property.
         *
         * @param id the value to set.
         */
        void
        setID (const std::string& id);

        /**
         * Get the LightSource property.
         *
         * @returns the LightSource property.
         */
        std::weak_ptr<ome::xml::model::LightSource>
        getLightSource ();

        /**
         * Get the LightSource property.
         *
         * @returns the LightSource property.
         */
        const std::weak_ptr<ome::xml::model::LightSource>
        getLightSource () const;

        /**
         * Set the LightSource property.
         *
         * @param lightSource the value to set.
         */
        void
        setLightSource (std::weak_ptr<ome::xml::model::LightSource>& lightSource);

        /**
         * Get the MicrobeamManipulation property.
         *
         * @returns the MicrobeamManipulation property.
         */
        std::weak_ptr<ome::xml::model::MicrobeamManipulation>
        getMicrobeamManipulation ();

        /**
         * Get the MicrobeamManipulation property.
         *
         * @returns the MicrobeamManipulation property.
         */
        const std::weak_ptr<ome::xml::model::MicrobeamManipulation>
        getMicrobeamManipulation () const;

        /**
         * Set the MicrobeamManipulation property.
         *
         * @param microbeamManipulation_BackReference the value to set.
         */
        void
        setMicrobeamManipulation (std::weak_ptr<ome::xml::model::MicrobeamManipulation>& microbeamManipulation_BackReference);

      protected:
        // Documented in base class.
        virtual void
        asXMLElementInternal (common::xml::dom::Document& document,
                              common::xml::dom::Element&  element) const;

      public:
       // Documented in superclass.
       const std::string&
       getXMLNamespace() const;
      };

    }
  }
}

#endif // OME_XML_MODEL_LIGHTSOURCESETTINGS_H

/*
 * Local Variables:
 * mode:C++
 * End:
 */
