/*
 * #%L
 * OME-XML C++ library for working with OME-XML metadata structures.
 * %%
 * Copyright © 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*─────────────────────────────────────────────────────────────────────────────
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *─────────────────────────────────────────────────────────────────────────────
 */

#ifndef OME_XML_MODEL_OBJECTIVE_H
#define OME_XML_MODEL_OBJECTIVE_H

#include <algorithm>
#include <list>
#include <stdexcept>
#include <string>
#include <vector>

#include <ome/common/log.h>

#include <ome/common/xml/dom/Document.h>
#include <ome/common/xml/dom/Element.h>
#include <ome/common/xml/dom/Node.h>

#include <ome/xml/model/primitives/Quantity.h>

#include <ome/xml/model/Instrument.h>
#include <ome/xml/model/ManufacturerSpec.h>
#include <ome/xml/model/enums/Correction.h>
#include <ome/xml/model/enums/Immersion.h>
#include <ome/xml/model/enums/UnitsLength.h>

namespace ome
{
  namespace xml
  {
    namespace model
    {

      // Forward declarations.
      class Annotation;
      class OMEModel;

      /**
       * Objective model object.
       */
      class Objective : public ManufacturerSpec
      {
      private:
        class Impl;
        /// Private implementation details.
        std::shared_ptr<Impl> impl;

      public:
        /// Default constructor.
        Objective();

        /**
         * Copy constructor.
         *
         * @param copy the Objective to copy.
         */
        Objective (const Objective& copy);

        /// Destructor.
        virtual
        ~Objective ();

        /**
         * Create a Objective model object from DOM element.
         *
         * @param element root of the XML DOM tree to from which to
         * construct the model object graph.
         * @param model handler for the OME model used to track
         * instances and references seen during the update.
         * @throws EnumerationException if there is an error
         * instantiating an enumeration during model object creation,
         * or ModelException if there are any consistency or validity
         * errors found during processing.
         *
         * @returns a new model object.
         */
        static std::shared_ptr<Objective>
        create(const common::xml::dom::Element& element,
               ome::xml::model::OMEModel& model);

        // Documented in superclass.
        const std::string&
        elementName() const;

        // Documented in superclass.
        bool
        validElementName(const std::string& name) const;

      private:
        /// Assignment operator (deleted).
        Objective&
        operator= (const Objective&);

      public:
        // -- OMEModelObject API methods --

        /// @copydoc ome::xml::model::OMEModelObject::update
        virtual void
        update(const common::xml::dom::Element&  element,
               ome::xml::model::OMEModel& model);

      public:
        // -- Objective API methods --

        /// @copydoc ome::xml::model::OMEModelObject::link
        bool
        link (std::shared_ptr<Reference>& reference,
              std::shared_ptr<ome::xml::model::OMEModelObject>& object);

        /**
         * Get the Iris property.
         *
         * @returns the Iris property.
         */
        std::shared_ptr<bool>
        getIris ();

        /**
         * Get the Iris property.
         *
         * @returns the Iris property.
         */
        const std::shared_ptr<bool>
        getIris () const;

        /**
         * Set the Iris property.
         *
         * @param iris the value to set.
         */
        void
        setIris (std::shared_ptr<bool>& iris);

        /**
         * Get the WorkingDistance property.
         *
         * @returns the WorkingDistance property.
         */
        std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsLength > >&
        getWorkingDistance ();

        /**
         * Get the WorkingDistance property.
         *
         * @returns the WorkingDistance property.
         */
        const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsLength > >&
        getWorkingDistance () const;

        /**
         * Set the WorkingDistance property.
         *
         * @param workingDistance the value to set.
         */
        void
        setWorkingDistance (const std::shared_ptr<ome::xml::model::primitives::Quantity<ome::xml::model::enums::UnitsLength > >& workingDistance);

        /**
         * Get the Immersion property.
         *
         * @returns the Immersion property.
         */
        std::shared_ptr<ome::xml::model::enums::Immersion>
        getImmersion ();

        /**
         * Get the Immersion property.
         *
         * @returns the Immersion property.
         */
        const std::shared_ptr<ome::xml::model::enums::Immersion>
        getImmersion () const;

        /**
         * Set the Immersion property.
         *
         * @param immersion the value to set.
         */
        void
        setImmersion (std::shared_ptr<ome::xml::model::enums::Immersion>& immersion);

        /**
         * Get the Correction property.
         *
         * @returns the Correction property.
         */
        std::shared_ptr<ome::xml::model::enums::Correction>
        getCorrection ();

        /**
         * Get the Correction property.
         *
         * @returns the Correction property.
         */
        const std::shared_ptr<ome::xml::model::enums::Correction>
        getCorrection () const;

        /**
         * Set the Correction property.
         *
         * @param correction the value to set.
         */
        void
        setCorrection (std::shared_ptr<ome::xml::model::enums::Correction>& correction);

        /**
         * Get the LensNA property.
         *
         * @returns the LensNA property.
         */
        std::shared_ptr<double>
        getLensNA ();

        /**
         * Get the LensNA property.
         *
         * @returns the LensNA property.
         */
        const std::shared_ptr<double>
        getLensNA () const;

        /**
         * Set the LensNA property.
         *
         * @param lensNA the value to set.
         */
        void
        setLensNA (std::shared_ptr<double>& lensNA);

        /**
         * Get the NominalMagnification property.
         *
         * @returns the NominalMagnification property.
         */
        std::shared_ptr<double>
        getNominalMagnification ();

        /**
         * Get the NominalMagnification property.
         *
         * @returns the NominalMagnification property.
         */
        const std::shared_ptr<double>
        getNominalMagnification () const;

        /**
         * Set the NominalMagnification property.
         *
         * @param nominalMagnification the value to set.
         */
        void
        setNominalMagnification (std::shared_ptr<double>& nominalMagnification);

        /**
         * Get the CalibratedMagnification property.
         *
         * @returns the CalibratedMagnification property.
         */
        std::shared_ptr<double>
        getCalibratedMagnification ();

        /**
         * Get the CalibratedMagnification property.
         *
         * @returns the CalibratedMagnification property.
         */
        const std::shared_ptr<double>
        getCalibratedMagnification () const;

        /**
         * Set the CalibratedMagnification property.
         *
         * @param calibratedMagnification the value to set.
         */
        void
        setCalibratedMagnification (std::shared_ptr<double>& calibratedMagnification);

        /**
         * Get the ID property.
         *
         * @returns the ID property.
         */
        const std::string&
        getID () const;

        /**
         * Set the ID property.
         *
         * @param id the value to set.
         */
        void
        setID (const std::string& id);

        /**
         * Get size of linked Annotation list.
         *
         * @returns the size of the list.
         */
        OMEModelObject::indexed_container<ome::xml::model::Annotation, std::weak_ptr>::type::size_type
        sizeOfLinkedAnnotationList () const;

        /**
         * Get the linked Annotation list.
         *
         * @returns a reference to the list.
         */
        OMEModelObject::indexed_container<ome::xml::model::Annotation, std::weak_ptr>::type
        getLinkedAnnotationList () const;

        /**
         * Get linked Annotation.
         *
         * @param index the index number of the Annotation.
         * @returns a weak pointer to the Annotation.
         * @throws std::out_of_range if the index is invalid.
         */
        const std::weak_ptr<ome::xml::model::Annotation>&
        getLinkedAnnotation (OMEModelObject::indexed_container<ome::xml::model::Annotation, std::weak_ptr>::type::size_type index) const;

        /**
         * Set linked Annotation.
         *
         * @note The index must be valid.
         *
         * @param index the index number of the Annotation.
         * @param annotation the Annotation to set.
         * @returns a weak pointer to the Annotation.
         * @throws std::out_of_range if the index is invalid.
         */
        const std::weak_ptr<ome::xml::model::Annotation>&
        setLinkedAnnotation (OMEModelObject::indexed_container<ome::xml::model::Annotation, std::weak_ptr>::type::size_type index,
                                     const std::shared_ptr<ome::xml::model::Annotation>& annotation);

        /**
         * Link Annotation.
         *
         * @param annotation the Annotation to link.
         * @returns @c true if the object was added to the internal
         * annotationLinks list, otherwise @c false.
         *
         * @todo Why have a return value here; what is it used for?
         * Is this an artifact of the Java API?
         */
        bool
        linkAnnotation (const std::shared_ptr<ome::xml::model::Annotation>& annotation);

        /**
         * Unlink Annotation.
         *
         * @param annotation the Annotation to unlink.
         *
         * @returns @c true if the Annotation was unlinked, otherwise
         * @c false if the Annotation was not linked and could hence not be
         * unlinked.
         *
         * @todo Does the return value serve any useful purpose?  Is
         * this also inherited Java API?
         * @todo Doesn't the removal break the indexing?
         * @todo Does the removal deal with multiple instances; we
         * aren't preventing duplicates on insertion.
         */
        bool
        unlinkAnnotation (const std::shared_ptr<ome::xml::model::Annotation>& annotation);

        /**
         * Get the Instrument property.
         *
         * @returns the Instrument property.
         */
        std::weak_ptr<ome::xml::model::Instrument>
        getInstrument ();

        /**
         * Get the Instrument property.
         *
         * @returns the Instrument property.
         */
        const std::weak_ptr<ome::xml::model::Instrument>
        getInstrument () const;

        /**
         * Set the Instrument property.
         *
         * @param instrument_BackReference the value to set.
         */
        void
        setInstrument (std::weak_ptr<ome::xml::model::Instrument>& instrument_BackReference);

      protected:
        // Documented in base class.
        virtual void
        asXMLElementInternal (common::xml::dom::Document& document,
                              common::xml::dom::Element&  element) const;

      public:
       // Documented in superclass.
       const std::string&
       getXMLNamespace() const;
      };

    }
  }
}

#endif // OME_XML_MODEL_OBJECTIVE_H

/*
 * Local Variables:
 * mode:C++
 * End:
 */
