/*
 * #%L
 * OME-XML C++ library for working with OME-XML metadata structures.
 * %%
 * Copyright © 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*─────────────────────────────────────────────────────────────────────────────
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *─────────────────────────────────────────────────────────────────────────────
 */

#ifndef OME_XML_MODEL_ENUMS_NAMINGCONVENTION_H
#define OME_XML_MODEL_ENUMS_NAMINGCONVENTION_H

#include <ostream>
#include <map>
#include <string>

#include <ome/common/log.h>

#include <ome/xml/model/enums/EnumerationException.h>

// All values in the NamingConvention enumeration.
#define OME_XML_MODEL_ENUMS_NAMINGCONVENTION_VALUES (LETTER)(NUMBER)

namespace ome
{
  namespace xml
  {
    namespace model
    {
      namespace enums
      {

        /**
         * NamingConvention enumeration.
         */
        class NamingConvention
        {
        public:
          /// Enumeration values.
          enum enum_value
            {
              /**
              
            While the label type 'number' has a clear meaning the 'letter'
            type is more complex.

            If you have less than 26 values use letters A to Z.
            Once you get more than 26 values there are several different
            approaches in use.
            One we have see include:

            Single letter, then double letter each running A to Z, right first
            e.g. A, B, C, ... X, Y, Z, AA, AB, AC, ... AY, AZ, BA, BB, ...
            This is the format used by Microsoft Excel so users may be familiar with it.
            This is the approach we use in the OMERO client applications.

            CAPITALsmall, each running A to Z, small first
            e.g. Aa, Ab, Ac, ... Ax, Ay, Az, Ba, Bb, Bc, ... By, Bz, Ca, Cb, ...
            This is in use by some plate manufactures.

            Single letter, then double letter, then triple letter, and so on
            e.g. A, B, C, ... X, Y, Z, AA, BB, CC, ... YY, ZZ, AAA, BBB, ...
            This has the advantage that the first 26 are the same as the standard
            but has a problem an the labels get wider and wider leading to user
            interface problems.
          
              */
              LETTER,
              /**
              
            1, 2, 3, ...
          
              */
              NUMBER
            };

          /**
           * Construct a NamingConvention enumeration by an enumeration value.
           *
           * @param value the value of the enumeration.
           */
          NamingConvention (enum_value value);

          /**
           * Construct a NamingConvention enumeration by an enumeration name.
           *
           * Exact matching of the name will require the name to match
           * one of the valid enumeration names.  Inexact matching will
           * strip leading and trailing whitespace and perform a
           * case-insensitive match.  Exact matching is recommended for
           * use in program code which does not process user input;
           * inexact matching is recommended when processing
           * potentially malformed user input.
           *
           * @param name the name of the enumeration.
           * @param strict true to require an exact name, false to allow an inexact match.
           */
          NamingConvention (const std::string& name, bool strict = true);

          /**
           * Copy constructor.
           *
           * @param original the instance to copy.
           */
          NamingConvention (const NamingConvention& original);

          /**
           * Assignment operator.
           *
           */
          inline NamingConvention&
          operator=  (const NamingConvention& rhs)
          {
            this->value = rhs.value;
            this->name = rhs.name;
            return *this;
          }

          /**
           * Cast the enumeration to its value.
           *
           * @returns the enumeration value.
           */
          inline
          operator enum_value () const
          {
            return this->value;
          }

          /**
           * Cast the enumeration to its name.
           *
           * @returns the enumeration value.
           */
          inline
          operator const std::string& () const
          {
            return *this->name;
          }

          /// String map type.
          typedef std::map<std::string, NamingConvention::enum_value> string_map_type;
          /// Value map type.
          typedef std::map<NamingConvention::enum_value, std::string> value_map_type;

          /**
           * Get a map of valid string names and enum values.
           *
           * @returns a map of string names to enum values.
           */
          static const string_map_type&
          strings();

          /**
           * Get a map of valid enum values and string names.
           *
           * @returns a map of enum values to string names.
           */
          static const value_map_type&
          values();

        private:
          /**
           * Get a map of valid lowercased string names and enum values.
           *
           * @returns a map of lowercased string names to enum values.
           */
          static const string_map_type&
          lowercase_strings();

          /// Enumeration value.
          enum_value         value;
          /// Enumeration name.
          const std::string *name;
        };

        /**
         * Compare two NamingConvention objects for equality.
         *
         * @param lhs a NamingConvention object.
         * @param rhs a NamingConvention object.
         * @returns true if the enum values are the same, otherwise false.
         */
        inline bool
        operator== (const NamingConvention& lhs,
                    const NamingConvention& rhs)
        {
          return static_cast<NamingConvention::enum_value>(lhs) == static_cast<NamingConvention::enum_value>(rhs);
        }

        /**
         * Compare NamingConvention object and enum value for equality.
         *
         * @param lhs a NamingConvention object.
         * @param rhs a NamingConvention enum value.
         * @returns true if the enum values are the same, otherwise false.
         */
        inline bool
        operator== (const NamingConvention& lhs,
                    const NamingConvention::enum_value& rhs)
        {
          return static_cast<NamingConvention::enum_value>(lhs) == rhs;
        }

        /**
         * Compare enum value and NamingConvention for equality.
         *
         * @param lhs a NamingConvention enum value.
         * @param rhs a NamingConvention object.
         * @returns true if the enum values are the same, otherwise false.
         */
        inline bool
        operator== (const NamingConvention::enum_value& lhs,
                    const NamingConvention& rhs)
        {
          return lhs == static_cast<NamingConvention::enum_value>(rhs);
        }

        /**
         * Compare NamingConvention object and enum string value for equality.
         *
         * @param lhs a NamingConvention object.
         * @param rhs a NamingConvention enum string value.
         * @returns true if the enum values are the same, otherwise false.
         */
        inline bool
        operator== (const NamingConvention& lhs,
                    const std::string& rhs)
        {
          return static_cast<const std::string&>(lhs) == rhs;
        }

        /**
         * Compare enum string value and NamingConvention object for equality.
         *
         * @param lhs a NamingConvention enum string value.
         * @param rhs a NamingConvention object.
         * @returns true if the enum values are the same, otherwise false.
         */
        inline bool
        operator== (const std::string& lhs,
                    const NamingConvention& rhs)
        {
          return lhs == static_cast<const std::string&>(rhs);
        }

        /**
         * Compare two NamingConvention objects for non-equality.
         *
         * @param lhs a NamingConvention object.
         * @param rhs a NamingConvention object.
         * @returns true if the enum values are not the same, otherwise false.
         */
        inline bool
        operator!= (const NamingConvention& lhs,
                    const NamingConvention& rhs)
        {
          return static_cast<NamingConvention::enum_value>(lhs) != static_cast<NamingConvention::enum_value>(rhs);
        }

        /**
         * Compare NamingConvention object and enum value for non-equality.
         *
         * @param lhs a NamingConvention object.
         * @param rhs a NamingConvention enum value.
         * @returns true if the enum values are not the same, otherwise false.
         */
        inline bool
        operator!= (const NamingConvention& lhs,
                    const NamingConvention::enum_value& rhs)
        {
          return static_cast<NamingConvention::enum_value>(lhs) != rhs;
        }

        /**
         * Compare enum value and NamingConvention object for non-equality.
         *
         * @param lhs a NamingConvention enum value.
         * @param rhs a NamingConvention object.
         * @returns true if the enum values are not the same, otherwise false.
         */
        inline bool
        operator!= (const NamingConvention::enum_value& lhs,
                    const NamingConvention& rhs)
        {
          return lhs != static_cast<NamingConvention::enum_value>(rhs);
        }

        /**
         * Compare NamingConvention object and enum string value for non-equality.
         *
         * @param lhs a NamingConvention object.
         * @param rhs a NamingConvention enum string value.
         * @returns true if the enum values are not the same, otherwise false.
         */
        inline bool
        operator!= (const NamingConvention& lhs,
                    const std::string& rhs)
        {
          return static_cast<const std::string&>(lhs) != rhs;
        }

        /**
         * Compare enum string value and NamingConvention object for non-equality.
         *
         * @param lhs a NamingConvention enum string value.
         * @param rhs a NamingConvention object.
         * @returns true if the enum values are not the same, otherwise false.
         */
        inline bool
        operator!= (const std::string& lhs,
                    const NamingConvention& rhs)
        {
          return lhs != static_cast<const std::string&>(rhs);
        }

        /**
         * Output NamingConvention to output stream.
         *
         * @param os the output stream.
         * @param enumeration the NamingConvention to output.
         * @returns the output stream.
         */
        template<class charT, class traits>
        inline std::basic_ostream<charT,traits>&
        operator<< (std::basic_ostream<charT,traits>& os,
                    const NamingConvention& enumeration)
        {
          return os << static_cast<const std::string&>(enumeration);
        }

        /**
         * Set NamingConvention from input stream.
         *
         * @param is the input stream.
         * @param enumeration the NamingConvention to input.
         * @returns the input stream.
         */
        template<class charT, class traits>
        inline std::basic_istream<charT,traits>&
        operator>> (std::basic_istream<charT,traits>& is,
                    NamingConvention& enumeration)
        {
          std::string value;
          is >> value;
          if (is)
            {
              try
                {
                  enumeration = NamingConvention(value, false);
                }
              catch (const EnumerationException&)
                {
                  is.setstate(std::ios::failbit);
                }
            }

          return is;
        }

      }
    }
  }
}

#endif // OME_XML_MODEL_ENUMS_NAMINGCONVENTION_H

/*
 * Local Variables:
 * mode:C++
 * End:
 */
